/* minish.c - example of a minimalistic shell 
 * Comments in code deliberately omitted. :-)
 *
 * [ Lainattu Käyttöjärjestelmät -kurssille esimerkkinä shellin
 *   perusideasta ja Unixin fork() ja exec() -kutsujen käytöstä.
 *   Siirretty merkkijonon jäsentäminen omaan aliohjelmaansa, jotta
 *   pääohjelma näyttäisi selkeämmältä.
 *
 *   Alkuperäinen koodi on Tapani Tarvaisen kurssimateriaalista "UNIX
 *   ja shell-ohjelmointi". Tarkempia kommentteja koodissa ei ole,
 *   koska tämä on Tapanin kurssin tehtävämateriaalia ja perinteinen
 *   tenttikysymys myös Käyttöjärjestelmät -kurssilla (...esimerkiksi
 *   kommentoimaton koodi annetaan, ja toiminta on selitettävä tmv.)
 *   Tärkeä kohta ovat fork() ja execve() -kutsut: mitä ne tekevät,
 *   miten käyttöjärjestelmän tietorakenteet silloin muuttuvat, miten
 *   forkin paluuarvoa tulkitaan sovellusohjelmassa, ja millaiset syyt
 *   voivat johtaa minkäkin kutsun epäonnistumiseen!
 * 
 *   Periaate selitetty luentomonisteessa; tarkka totuus rajapinnasta
 *   löytyy esim. POSIXista ja todellinen toteutustapa esim. GCC:n
 *   C-kirjastoista ja Linuxin lähdekoodista. (Vain periaate ja
 *   sovellusrajapinta ovat johdantokurssin asiaa).
 *
 *   Terveisin Paavo 2007-06-05, 2014-04-01, 2015-04-23, 2016-04-27. ]
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/wait.h>
#define MAXLINE 50

/**
 * Suomennetaan kutsu C:n esikääntäjämakrolla :) Palauttaa toden, jos
 * syötteessä on lisää rivejä juuri luetun rivin jälkeen.
 */
#define lue_komentorivi(s, n, f) (fgets(s, n, f))

/**
 * Pilkkoo merkkijonon välilyönneistä. Palauttaa 0, jos eka merkki on
 * '#' ja muutoin 1. (vain esittely koodin alussa; toteutus saa olla
 * missä vain)
 */
int jasenna_komentorivi(char *line, char **args); 

/** 
 * Pääohjelma. Olennainen ymmärrettävä osuus merkitty erikseen alempana.
 */
int main(int argc, char **argv)
{
    char line[MAXLINE+1], *args[MAXLINE];

    if (argc>=2 && !freopen(argv[1], "r", stdin)) {
        perror(argv[1]);
        exit(errno);
    }

    /* TÄMÄ WHILE-SILMUKKA YMMÄRRETTÄVÄ KOKONAISUUDESSAAN: */
    while (lue_komentorivi(line, MAXLINE, stdin)) {
        if (!jasenna_komentorivi(line,args)) continue;
        
        if (!strcmp(line,"exit")) {
            exit(args[1] ? atoi(args[1]) : 0);
        }
                
        switch (fork()) {
        case 0: 
            execvp(args[0], args);
            perror(args[0]);
            exit(errno);
        case -1: 
            perror("fork() failed");
            break;
        default: 
            wait(NULL);
        } 
    }    
    exit(0);
}

/* Aiemmin esitellyn aliohjelman toteutus (ei olennainen KJ-kurssilla). */
int jasenna_komentorivi(char *line, char **args){
    char *p, **arg;
    if ((p = strchr(line, '\n'))) {
        *p=0; 
    } else { 
        fprintf(stderr, "line too long\n"); 
        exit(1); 
    }
    p = line;
    while (*p == ' ') 
        ++p;
    if (*p == '#') 
        return 0;
    *(arg=args) = p;           
    while ((p = strchr(p, ' '))) {
        while (*p == ' ') 
            *p++ = 0;
        if (*p) 
            *++arg=p;
    }
    *++arg=NULL;
    return 1;
}
